@@ -1,7 +1,7 @@
 /* Cookbooks! The ultimate building block of any chef run. */
 
 /*
- * Copyright (c) 2013-2016, Jeremy Bingham (<jeremy@goiardi.gl>)
+ * Copyright (c) 2013-2017, Jeremy Bingham (<jeremy@goiardi.gl>)
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -267,6 +267,24 @@ func Get(name string) (*Cookbook, util.Gerror) {
 	return cookbook, nil
 }
 
+// DoesExist checks if the cookbook in question exists or not
+func DoesExist(cookbookName string) (bool, util.Gerror) {
+	var found bool
+	if config.UsingDB() {
+		var cerr error
+		found, cerr = checkForCookbookSQL(datastore.Dbh, cookbookName)
+		if cerr != nil {
+			err := util.Errorf(cerr.Error())
+			err.SetStatus(http.StatusInternalServerError)
+			return false, err
+		}
+	} else {
+		ds := datastore.New()
+		_, found = ds.Get("cookbook", cookbookName)
+	}
+	return found, nil
+}
+
 // Save a cookbook to the in-memory data store or database.
 func (c *Cookbook) Save() error {
 	var err error
@@ -851,6 +869,27 @@ func (c *Cookbook) GetVersion(cbVersion string) (*CookbookVersion, util.Gerror)
 	return cbv, nil
 }
 
+// DoesVersionExist checks if a particular version of a cookbook exists
+func (c *Cookbook) DoesVersionExist(cbVersion string) (bool, util.Gerror) {
+	if config.UsingDB() {
+		found, err := c.checkCookbookVersionSQL(cbVersion)
+		if err != nil {
+			cerr := util.CastErr(err)
+			return false, cerr
+		}
+		return found, nil
+	}
+	cbv, err := c.GetVersion(cbVersion)
+	if err != nil {
+		return false, err
+	}
+	var found bool
+	if cbv != nil {
+		found = true
+	}
+	return found, nil
+}
+
 func extractVerNums(cbVersion string) (maj, min, patch int64, err util.Gerror) {
 	if _, err = util.ValidateAsVersion(cbVersion); err != nil {
 		return 0, 0, 0, err
@@ -911,7 +950,7 @@ func (c *Cookbook) deleteHashes(fhashes []string) {
 					 * one in fhashes, also break out.
 					 */
 					if fh == vh {
-						fhashes = delSliceElement(i, fhashes)
+						fhashes = util.DelSliceElement(i, fhashes)
 						break
 					} else if fh > vh {
 						break
@@ -1231,23 +1270,9 @@ func getAttrHashes(attr []map[string]interface{}) []string {
 }
 
 func removeDupHashes(fhashes []string) []string {
-	for i, v := range fhashes {
-		/* break if we're the last element */
-		if i+1 >= len(fhashes) {
-			break
-		}
-		/* If the current element is equal to the next one, remove
-		 * the next element. */
-		if v == fhashes[i+1] {
-			fhashes = delSliceElement(i+1, fhashes)
-		}
-	}
-	return fhashes
-}
-
-func delSliceElement(pos int, strs []string) []string {
-	strs = append(strs[:pos], strs[pos+1:]...)
-	return strs
+	// needed this functionality elsewhere, so it's been moved to util.
+	// Keeping this as a wrapper for simplicity.
+	return util.RemoveDupStrings(fhashes)
 }
 
 // RecipeList provides a list of recipes in this cookbook version.
