@@ -2,7 +2,7 @@
  * to learn more about programming in Go. */
 
 /*
- * Copyright (c) 2013-2016, Jeremy Bingham (<jeremy@goiardi.gl>)
+ * Copyright (c) 2013-2017, Jeremy Bingham (<jeremy@goiardi.gl>)
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -21,6 +21,7 @@ package main
 
 import (
 	"compress/gzip"
+	"context"
 	"crypto/tls"
 	"encoding/gob"
 	"encoding/json"
@@ -51,6 +52,7 @@ import (
 	"github.com/ctdk/goiardi/loginfo"
 	"github.com/ctdk/goiardi/node"
 	"github.com/ctdk/goiardi/report"
+	"github.com/ctdk/goiardi/reqctx"
 	"github.com/ctdk/goiardi/role"
 	"github.com/ctdk/goiardi/sandbox"
 	"github.com/ctdk/goiardi/search"
@@ -73,6 +75,14 @@ type apiTimerInfo struct {
 	method  string
 }
 
+var noOpUserReqs = []string{
+	"/authenticate_user",
+	"/file_store",
+	"/universe",
+	"/principals",
+	"/debug",
+}
+
 var apiChan chan *apiTimerInfo
 
 func main() {
@@ -179,7 +189,7 @@ func main() {
 			os.Exit(1)
 		}
 		errch := make(chan error)
-		go startEventMonitor(serfin.Serfer, errch)
+		go startEventMonitor(config.Config.SerfAddr, errch)
 		err := <-errch
 		if err != nil {
 			logger.Criticalf(err.Error())
@@ -188,6 +198,14 @@ func main() {
 		startNodeMonitor()
 	}
 
+	if config.Config.PurgeNodeStatusAfter != "" {
+		startNodeStatusPurge()
+	}
+
+	if config.Config.PurgeReportsAfter != "" {
+		startReportPurge()
+	}
+
 	/* Create default clients and users. Currently chef-validator,
 	 * chef-webui, and admin. */
 	createDefaultActors()
@@ -299,12 +317,9 @@ func (h *interceptHandler) ServeHTTP(w http.ResponseWriter, r *http.Request) {
 			if len(fwded) != 0 {
 				xForwarded = fwded[len(fwded)-1]
 			}
-			logger.Debugf("remote ip candidates: ra: '%s', '%s'", remoteIP, xForwarded)
 			rIP := net.ParseIP(remoteIP)
 			xFIP := net.ParseIP(xForwarded)
-			logger.Debugf("ips now: '%q' '%q'", rIP, xFIP)
-			logger.Debugf("local? '%q' '%q'", rIP.IsLoopback(), xFIP.IsLoopback())
-			if !rIP.IsLoopback() && !xFIP.IsLoopback() {
+			if !rIP.IsLoopback() && !xFIP.IsLoopback() && !config.PprofWhitelisted(rIP) && !config.PprofWhitelisted(xFIP) {
 				logger.Debugf("blocked %s (x-forwarded-for: %s) from accessing /debug/pprof!", rIP.String(), xFIP.String())
 				block = true
 			}
@@ -370,13 +385,12 @@ func (h *interceptHandler) ServeHTTP(w http.ResponseWriter, r *http.Request) {
 	/* Only perform the authorization check if that's configured. Bomb with
 	 * an error if the check of the headers, timestamps, etc. fails. */
 	/* No clue why /principals doesn't require authorization. Hrmph. */
-	if config.Config.UseAuth && !strings.HasPrefix(r.URL.Path, "/file_store") && !(strings.HasPrefix(r.URL.Path, "/principals") && r.Method == "GET") {
+	if config.Config.UseAuth && !strings.HasPrefix(r.URL.Path, "/file_store") && !strings.HasPrefix(r.URL.Path, "/debug") && !(strings.HasPrefix(r.URL.Path, "/principals") && r.Method == "GET") {
 		herr := authentication.CheckHeader(userID, r)
 		if herr != nil {
 			w.Header().Set("Content-Type", "application/json")
 			logger.Errorf("Authorization failure: %s\n", herr.Error())
-			w.Header().Set("Www-Authenticate", `X-Ops-Sign version="1.0" version="1.1" version="1.2"`)
-			//http.Error(w, herr.Error(), herr.Status())
+			w.Header().Set("Www-Authenticate", `X-Ops-Sign version="1.0" version="1.1" version="1.2" version="1.3"`)
 			jsonErrorReport(w, r, herr.Error(), herr.Status())
 			return
 		}
@@ -394,7 +408,29 @@ func (h *interceptHandler) ServeHTTP(w http.ResponseWriter, r *http.Request) {
 		r.Body = reader
 	}
 
-	http.DefaultServeMux.ServeHTTP(w, r)
+	// Set up the context for the request. At this time, this means setting
+	// the opUser for this request for most (but not all) types of requests.
+	// At this time the exceptions are "/file_store", "/universe", and
+	// "/authenticate_user".
+	ctx := r.Context()
+	var skip bool
+	for _, p := range noOpUserReqs {
+		if strings.HasPrefix(r.URL.Path, p) {
+			skip = true
+			break
+		}
+	}
+	if !skip {
+		opUser, oerr := actor.GetReqUser(r.Header.Get("X-OPS-USERID"))
+		if oerr != nil {
+			w.Header().Set("Content-Type", "application/json")
+			jsonErrorReport(w, r, oerr.Error(), oerr.Status())
+			return
+		}
+		ctx = context.WithValue(ctx, reqctx.OpUserKey, opUser)
+	}
+
+	http.DefaultServeMux.ServeHTTP(w, r.WithContext(ctx))
 }
 
 func cleanPath(p string) string {
@@ -526,7 +562,7 @@ func handleSignals() {
 					datastore.Dbh.Close()
 				}
 				if config.Config.UseSerf {
-					serfin.Serfer.Close()
+					serfin.CloseAll()
 				}
 				os.Exit(0)
 			} else if sig == syscall.SIGHUP {
@@ -631,54 +667,110 @@ func setLogEventPurgeTicker() {
 	}
 }
 
-func startEventMonitor(sc *serfclient.RPCClient, errch chan<- error) {
-	ch := make(chan map[string]interface{}, 10)
-	sh, err := sc.Stream("*", ch)
+// The serf functionality needs some cleaning up. This is a start on that.
+func startEventMonitor(serfAddr string, errch chan<- error) {
+	// Initial setup of serf. If this bombs go ahead and return so we can
+	// die
+	sc, err := serfin.NewRPCClient(serfAddr)
 	if err != nil {
 		errch <- err
 		return
 	}
 	errch <- nil
 
-	defer sc.Stop(sh)
-	// watch the events and queries
-	for e := range ch {
-		logger.Debugf("Got an event: %v", e)
-		eName, _ := e["Name"]
-		switch eName {
-		case "node_status":
-			jsonPayload := make(map[string]string)
-			err = json.Unmarshal(e["Payload"].([]byte), &jsonPayload)
+	ech := make(chan error)
+	recreateSerfWait := time.Duration(5)
+
+	for {
+		// Make sure the serf client is actually closed before creating
+		// a new one. The very first time this loop is kicked off, of
+		// course, the client will be fine. It's simpler to have the
+		// check up here, though, rather than at the end
+		if sc == nil || sc.IsClosed() {
+			sc, err = serfin.NewRPCClient(serfAddr)
 			if err != nil {
-				logger.Errorf(err.Error())
+				logger.Errorf("Error recreating serf client, waiting %d seconds before recreating: %s", recreateSerfWait, err.Error())
+				time.Sleep(recreateSerfWait * time.Second)
 				continue
+			} else {
+				logger.Errorf("reconnected to serf after being disconnected")
+			}
+		}
+		go runEventMonitor(sc, ech)
+		e := <-ech
+		if e != nil {
+			logger.Errorf("Error from event monitor: %s", e.Error())
+		}
+	}
+}
+
+func runEventMonitor(sc *serfclient.RPCClient, errch chan<- error) {
+	ch := make(chan map[string]interface{}, 10)
+	sh, err := sc.Stream("*", ch)
+	if err != nil {
+		errch <- err
+		return
+	}
+
+	defer sc.Stop(sh)
+	checkClientSec := time.Duration(15)
+
+	// watch the events and queries
+	for {
+		select {
+		case e := <-ch:
+			eNil := e == nil
+			logger.Debugf("Got an event: %v nil? %v", e, eNil)
+			if eNil {
+				if sc.IsClosed() {
+					logger.Debugf("Serf client has been closed, returning from runEventMonitor in hopes of being able to reconnect")
+					err := fmt.Errorf("serf client closed")
+					errch <- err
+					return
+				}
+				continue
+			}
+			eName, _ := e["Name"]
+			switch eName {
+			case "node_status":
+				jsonPayload := make(map[string]string)
+				err = json.Unmarshal(e["Payload"].([]byte), &jsonPayload)
+				if err != nil {
+					logger.Errorf(err.Error())
+					continue
+				}
+				n, _ := node.Get(jsonPayload["node"])
+				if n == nil {
+					logger.Errorf("No node %s", jsonPayload["node"])
+					continue
+				}
+				err = n.UpdateStatus(jsonPayload["status"])
+				if err != nil {
+					logger.Errorf(err.Error())
+					continue
+				}
+				r := map[string]string{"response": "ok"}
+				response, _ := json.Marshal(r)
+				var id uint64
+				switch t := e["ID"].(type) {
+				case int64:
+					id = uint64(t)
+				case uint64:
+					id = t
+				default:
+					logger.Errorf("node_status ID %v type %T not int64 or uint64", e["ID"], e["ID"])
+					continue
+				}
+				sc.Respond(id, response)
+			}
+		case <-time.After(checkClientSec * time.Second):
+			if sc.IsClosed() {
+				clerr := fmt.Errorf("serf client found to be closed, recreating")
+				errch <- clerr
+				return
 			}
-			n, _ := node.Get(jsonPayload["node"])
-			if n == nil {
-				logger.Errorf("No node %s", jsonPayload["node"])
-				continue
-			}
-			err = n.UpdateStatus(jsonPayload["status"])
-			if err != nil {
-				logger.Errorf(err.Error())
-				continue
-			}
-			r := map[string]string{"response": "ok"}
-			response, _ := json.Marshal(r)
-			var id uint64
-			switch t := e["ID"].(type) {
-			case int64:
-				id = uint64(t)
-			case uint64:
-				id = t
-			default:
-				logger.Errorf("node_status ID %v type %T not int64 or uint64", e["ID"], e["ID"])
-				continue
-			}
-			sc.Respond(id, response)
 		}
 	}
-	return
 }
 
 func startNodeMonitor() {
@@ -709,6 +801,40 @@ func startNodeMonitor() {
 	return
 }
 
+func startReportPurge() {
+	go func() {
+		// purge reports after 2 hours, I guess.
+		ticker := time.NewTicker(2 * time.Hour)
+		for _ = range ticker.C {
+			del, err := report.DeleteByAge(config.Config.PurgeReportsDur)
+			if err != nil {
+				logger.Errorf("Purging reports had an error: %s", err.Error())
+			} else {
+				logger.Debugf("Purged %d reports", del)
+			}
+		}
+	}()
+}
+
+func startNodeStatusPurge() {
+	// don't do it if there aren't going to be node statuses to purge
+	if !config.Config.UseSerf || config.Config.PurgeNodeStatusDur == 0 {
+		return
+	}
+	go func() {
+		// check every 2 hours for statuses to purge
+		ticker := time.NewTicker(2 * time.Hour)
+		for _ = range ticker.C {
+			del, err := node.DeleteNodeStatusesByAge(config.Config.PurgeNodeStatusDur)
+			if err != nil {
+				logger.Errorf("Purging node statuses had an error: %s", err.Error())
+			} else {
+				logger.Debugf("Purged %d node statuses", del)
+			}
+		}
+	}()
+}
+
 func initGeneralStatsd(metricsBackend met.Backend) {
 	if !config.Config.UseStatsd {
 		return
