@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2013-2016, Jeremy Bingham (<jeremy@goiardi.gl>)
+ * Copyright (c) 2013-2017, Jeremy Bingham (<jeremy@goiardi.gl>)
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -167,7 +167,7 @@ func New(command string, timeout int, quorumStr string, nodeNames []string) (*Sh
 	// Conflicting uuids are unlikely, but conceivable.
 	if config.UsingDB() {
 		var err error
-		found, err = checkForShoveySQL(datastore.Dbh, runID)
+		found, err = checkForShoveySQL(runID)
 		if err != nil {
 			gerr := util.CastErr(err)
 			gerr.SetStatus(http.StatusInternalServerError)
@@ -289,6 +289,21 @@ func Get(runID string) (*Shovey, util.Gerror) {
 	return shove, nil
 }
 
+// DoesExist checks if there is a shovey instance with the given run id.
+func DoesExist(runID string) (bool, util.Gerror) {
+	if config.UsingDB() {
+		found, err := checkForShoveySQL(runID)
+		if err != nil {
+			serr := util.CastErr(err)
+			return false, serr
+		}
+		return found, nil
+	}
+	ds := datastore.New()
+	_, found := ds.Get("shovey", runID)
+	return found, nil
+}
+
 // Cancel cancels all ShoveyRuns associated with this shovey instance.
 func (s *Shovey) Cancel() util.Gerror {
 	err := s.CancelRuns(s.NodeNames)
@@ -351,7 +366,10 @@ func (s *Shovey) CancelRuns(nodeNames []string) util.Gerror {
 	jsonPayload, _ := json.Marshal(payload)
 	ackCh := make(chan string, len(nodeNames))
 	q := &serfclient.QueryParam{Name: "shovey", Payload: jsonPayload, FilterNodes: nodeNames, RequestAck: true, AckCh: ackCh}
-	err := serfin.Serfer.Query(q)
+	errch := make(chan error)
+	go serfin.Query(q, errch)
+	err := <-errch
+
 	if err != nil {
 		return util.CastErr(err)
 	}
@@ -435,12 +453,16 @@ func (s *Shovey) startJobs() Qerror {
 		jsonPayload, _ := json.Marshal(payload)
 		ackCh := make(chan string, len(tagNodes))
 		respCh := make(chan serfclient.NodeResponse, len(tagNodes))
+
 		q := &serfclient.QueryParam{Name: "shovey", Payload: jsonPayload, FilterNodes: tagNodes, RequestAck: true, AckCh: ackCh, RespCh: respCh}
-		qerr := serfin.Serfer.Query(q)
+		qerrch := make(chan error)
+		go serfin.Query(q, qerrch)
+		qerr := <-qerrch
 		if qerr != nil {
 			errch <- qerr
 			return
 		}
+
 		errch <- nil
 		srCh := make(chan *ShoveyRun, len(upNodes)*2)
 
